%{
/****************************************************************/
/* Parser specification for Integration Framework configuration */
/* control file                                                 */
/****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "config.h"
#include "variables.h"

/***************************************************************/
/* Code to handle the creation of named variables and their    */
/* subsequent retrieval at runtime from the application        */
/***************************************************************/

float configGetFloat(char *named)
{
userVarPtrT varPtr = configFile.UserVars;
while (varPtr != NULL)
  {
  if ((strcmp(varPtr->name, named) == 0) &&
      (varPtr->type == USER_VAR_FLOAT_TYPE))
    {
#ifdef DEBUG
  fprintf(stderr, "GETFLOATVAR (%s) found %f\n", named, varPtr->value.f);
#endif

    return varPtr->value.f;
    }
  varPtr = varPtr->next;
  }
#ifdef DEBUG
  fprintf(stderr, "GETFLOATVAR (%s) not found\n", named);
#endif
return 0.0;
}

/***************************************************************/
long configGetInt(char *named)
{
userVarPtrT varPtr = configFile.UserVars;
while (varPtr != NULL)
  {
  if ((strcmp(varPtr->name, named) == 0) &&
      (varPtr->type == USER_VAR_INTEGER_TYPE))
    {
#ifdef DEBUG
  fprintf(stderr, "GETINTEGERVAR (%s) found %ld\n", named, varPtr->value.i);
#endif

    return varPtr->value.i;
    }
  varPtr = varPtr->next;
  }
#ifdef DEBUG
  fprintf(stderr, "GETINTEGERVAR (%s) not found\n", named);
#endif
return 0L;
}

/***************************************************************/
char *configGetString(char *named)
{
userVarPtrT varPtr = configFile.UserVars;
while (varPtr != NULL)
  {
  if ((strcmp(varPtr->name, named) == 0) &&
      (varPtr->type == USER_VAR_STRING_TYPE))
    {
#ifdef DEBUG
  fprintf(stderr, "GETSTRINGVAR (%s) found (%s)\n", named, varPtr->value.s);
#endif

    return varPtr->value.s;
    }
  varPtr = varPtr->next;
  }
#ifdef DEBUG
  fprintf(stderr, "GETSTRINGVAR (%s) not found\n", named);
#endif
return NULL;
}

/***************************************************************/
static void insertVariable(userVarPtrT varPtr)
{
/* Simple push on link */
varPtr->next        = configFile.UserVars;
configFile.UserVars = varPtr;
}

/***************************************************************/
static userVarPtrT createVariable(char       *named,
                                  userValueT  ofType)
{
/* Get the length of the name */
int nLen = strlen(named);

userVarPtrT varPtr = (userVarPtrT )calloc(1, sizeof(userVarT));

/* Copy the name in to the variable */
varPtr->name = (char *)calloc(1, nLen + 1);
strncpy(varPtr->name, named, nLen);

/* Set the type */
varPtr->type = ofType;

insertVariable(varPtr);
return varPtr;
}

/***************************************************************/

int  StringLength;
long DefaultVal;

/* The following variables are used to  */
/* provide a string to print in yyerror */
/* to say what was expected next. They  */
/* are printed from within the lexer    */

static char *firstLevel     = "variable name, type specifier, "
                              "or the keyword WARNINGS_ALLOWED";
static char *Equals         = "assignment operator =";
static char *String         = "text surrounded by double quotes";
static char *VarString      = "user variable name";
static char *NewLine        = "statement separator, the new line character";
static char *DefaultNum     = "a number, or the keyword DEFAULT";
static char *Number         = "an integer number";
static char *FloatNumber    = "a floating point number";
static char *DefaultString  = "text surrounded by double quotes, "
                              "or the keyword DEFAULT";
static char *nextExpected;

/* Variable used by lexer to print  */
/* description of what was expected */
/* next, used in yyerror            */

char *expectedString = "variable name, type specifier, "
                       "or the keyword WARNINGS_ALLOWED";
%}

%union {
	char*		string;
	char**		charAddr;
	long		longInt;
	long*		longAddr;
	float		floatNum;
	userVarPtrT	variable;
	}

%token	<string>	DT_EQUALS
%token	<string>	DT_TEXT
%token	<string>	DT_NEWLINE
%token	<longInt>	DT_INTEGER
%token	<floatNum>	DT_FLOAT
%token	<string>	DT_VARNAME
%token	<string>	DT_STRING
%token	<string>	DT_NULLSTRING
%token	<string>	DT_STRING_VAR
%token	<string>	DT_INTEGER_VAR
%token	<string>	DT_FLOAT_VAR
%token	<string>	DT_QMANAGER
%token	<string>	DT_QNAME
%token	<string>	DT_QMSGSIZE
%token	<string>	DT_SFMFLV
%token	<string>	DT_DEFAULT
%token	<string>	DT_SFMIDFILE
%token	<string>	DT_RQNAME
%token	<string>	DT_ERRQNAME
%token	<string>	DT_ERRFNAME
%token	<string>	DT_MAILADDR
%token	<string>	DT_WARNINGS
%token	<string>	DT_SOURCEREF
%token	<string>	DT_SLEEPTIME
%token	<string>	DT_DBLOGIN
%token	<string>	DT_ACKIDFILE
%token	<string>	DT_FILEDIR
%token	<string>	DT_FILELEAF

%type	<charAddr>	CharVar
%type	<string>	CharString
%type	<charAddr>	DefCharVar
%type	<string>	DefaultString
%type	<longAddr>	LongVar
%type	<longInt>	LongNum
%type	<longAddr>	DefLongVar
%type	<longInt>	DefLongNum
%type	<floatNum>	FloatNum
%type	<variable>	UserString
%type	<variable>	UserInteger
%type	<variable>	UserFloat

%%

statements:	  statement
		| statements statement
		;

statement:	  assignment separator
                | WarningsOK separator
                | userVar separator
		| separator
                ;

separator:	  DT_NEWLINE { expectedString = firstLevel; }
		;

assignment:	  CharVar Equals CharString {
			*$1 = (char *)calloc(1, StringLength);
			strncpy(*$1, $3 + 1, StringLength - 1);
		  }
		| DefCharVar Equals DefaultString {
			*$1 = (char *)calloc(1, StringLength);
			strncpy(*$1, $3 + 1, StringLength - 1);
		  }
		| LongVar Equals LongNum { *$1 = $3; }
		| DefLongVar Equals DefLongNum { *$1 = $3; }
                ;

userVar:	  UserString Equals CharString {
			$1->value.s = (char *)calloc(1, StringLength);
			strncpy($1->value.s, $3 + 1, StringLength - 1);
		  }
		| UserInteger Equals LongNum { $1->value.i = $3; }
		| UserFloat Equals FloatNum { $1->value.f = $3; }
		;

UserString:	  StringVar DT_VARNAME {
			$$ = createVariable($2, USER_VAR_STRING_TYPE);
			expectedString = Equals;
			nextExpected   = String;
		  }
		;

UserInteger:	  IntegerVar DT_VARNAME {
			$$ = createVariable($2, USER_VAR_INTEGER_TYPE);
			expectedString = Equals;
			nextExpected   = Number;
		  }
		;

UserFloat:	  FloatVar DT_VARNAME {
			$$ = createVariable($2, USER_VAR_FLOAT_TYPE);
			expectedString = Equals;
			nextExpected   = FloatNumber;
		  }
		;

StringVar:	  DT_STRING_VAR { expectedString = VarString; }
		;

FloatVar:	  DT_FLOAT_VAR { expectedString = VarString; }
		;

IntegerVar:	  DT_INTEGER_VAR { expectedString = VarString; }
		;

Equals:		  DT_EQUALS { expectedString = nextExpected; }
		;

CharVar:	  DT_QNAME {
			$$ = &(configFile.QName);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_RQNAME {
			$$ = &(configFile.RQName);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_ERRQNAME {
			$$ = &(configFile.ErrQName);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_ERRFNAME {
			$$ = &(configFile.ErrFileName);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_MAILADDR {
			$$ = &(configFile.EMailAddress);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_SOURCEREF {
			$$ = &(configFile.SourceRef);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_SFMIDFILE {
			$$ = &(configFile.SFMLastIdFile);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_ACKIDFILE {
			$$ = &(configFile.AckIdFile);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_FILEDIR {
			$$ = &(configFile.FileDirectory);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_FILELEAF {
			$$ = &(configFile.FileLeafName);
			expectedString = Equals;
			nextExpected   = String;
		  }
		| DT_DBLOGIN {
			$$ = &(configFile.DatabaseLogin);
			expectedString = Equals;
			nextExpected   = String;
		  }
		;

CharString:	  DT_STRING {
			StringLength = strlen($1) - 1;
			expectedString = NewLine;
		  }
		| DT_NULLSTRING {
			$$ = "";
			StringLength = 1;
			expectedString = NewLine;
		  }
		;

LongVar:	  DT_SLEEPTIME {
			$$ = &(configFile.SleepTime);
			expectedString = Equals;
			nextExpected   = Number;
		  }
		;

LongNum:	  DT_INTEGER { expectedString = NewLine; }
		;

FloatNum:	  DT_FLOAT { expectedString = NewLine; }
		;

DefLongVar:	  DT_QMSGSIZE {
			$$ = &(configFile.QMessageSize);
			DefaultVal = configFile.DefaultQMessageSize;
			expectedString = Equals;
			nextExpected   = DefaultNum;
		  }
		| DT_SFMFLV {
			$$ = &(configFile.SFMFlavour);
			DefaultVal = configFile.DefaultSFMFlavour;
			expectedString = Equals;
			nextExpected   = DefaultNum;
		  }
		;

DefLongNum:	  DT_INTEGER { expectedString = NewLine; }
		| DT_DEFAULT { $$ = DefaultVal; expectedString = NewLine; }
		;

DefCharVar:	  DT_QMANAGER {
			$$ = &(configFile.QManager);
			expectedString = Equals;
			nextExpected   = DefaultString;
		  }
		;

DefaultString:	  DT_STRING {
			StringLength = strlen($1) - 1;
			expectedString = NewLine;
		  }
		| DT_NULLSTRING {
			$$ = "";
			StringLength = 1;
			expectedString = NewLine;
		  }
		| DT_DEFAULT {
			$$ = "";
			StringLength = 1;
			expectedString = NewLine;
		  }
		;

WarningsOK:	  DT_WARNINGS {
			configFile.WarningsAllowed = 1;
			expectedString = NewLine;
		  }
		;
